package utils;

import java.util.Iterator;

import javax.faces.FactoryFinder;
import javax.faces.application.Application;
import javax.faces.application.ApplicationFactory;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;


/**
   Utility class with static methods for accessing different 
   features of JSF.
**/
public class FacesUtils {

	/**
	 * Get servlet context.
	 *
	 * @return the servlet context
	 */
	public static ServletContext getServletContext()
	{
		return (ServletContext)FacesContext.getCurrentInstance().getExternalContext().getContext();
	}

	/**
	   Returns the web application context.
	   @return the web application context. 
	**/
	public static String getWebContext()
	{
		return FacesContext.getCurrentInstance().getExternalContext().getRequestContextPath();
	}

	/**
	 * Get managed bean based on the bean name.
	 *
	 * @param beanName the bean name
	 * @return the managed bean associated with the bean name
	 */
	public static Object getManagedBean(String beanName)
	{
		Object o = getValueBinding(getJsfEl(beanName)).getValue(FacesContext.getCurrentInstance());

		return o;
	}

    /**
     * Using the given view id, figure out the name of the corresponding
     * managed bean.
     * 
     * @param viewId the view id to use when building the managed bean name
     */
    public static String getManagedBeanNameFromView(String viewId)
    {
        if (viewId == null)
            return null;

        return getManagedPrefix(viewId) + "BackingBean";
    }
    
    /**
     * Using the given view id, figure out the name of the corresponding
     * managed bean prefix.
     * 
     * @param viewId the view id to use when building the managed bean name
     */
    public static String getManagedPrefix(String viewId)
    {
        if (viewId == null)
            return null;
        
        int slashIndex = viewId.lastIndexOf('/');
        if (slashIndex >= 0)
            viewId = viewId.substring(slashIndex + 1);
        
        int dotIndex = viewId.lastIndexOf('.');
        if (dotIndex >= 0)
            viewId = viewId.substring(0, dotIndex);
        
        return viewId;
    }
    
	/**
	 * Remove the managed bean based on the bean name.
	 *
	 * @param beanName the bean name of the managed bean to be removed
	 */
	public static void resetManagedBean(String beanName)
	{
		getValueBinding(getJsfEl(beanName)).setValue(FacesContext.getCurrentInstance(), null);
	}

	/**
	 * Store the managed bean inside the session scope.
	 *
	 * @param beanName the name of the managed bean to be stored
	 * @param managedBean the managed bean to be stored
	 */
	public static void setManagedBeanInSession(String beanName, Object managedBean)
	{
		FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put(beanName, managedBean);
	}

	/**
	   Returns the object the component is bound to or null if not found.
	   @param context the Faces context.  
	   @param component the component to get the object from.
	**/
	public static Object getComponentObject(UIComponent component)
	{
		// Get the value binding expression for the output field
		ValueBinding vb = component.getValueBinding("value");
		String expr = vb.getExpressionString();
		
		// If it isn't an expression then bail
		if (expr == null)
			return null;
		int propIdx = expr.lastIndexOf(".");
		if (propIdx <= 0)
			return null;

		// Build the object expression.
		String objExpr = expr.substring(0, propIdx) + "}";
		String propName = expr.substring(propIdx + 1, (expr.length() - 1));
		
		// Get the object holding this property.
		Object o = FacesUtils.getValueBinding(objExpr).getValue(FacesContext.getCurrentInstance());
		return o;
	}

	/**
	   Get the property name from the expression.
	**/
	public static String getComponentProperty(UIComponent component)
	{
		ValueBinding vb = component.getValueBinding("value");
		String expr = vb.getExpressionString();
		if (expr == null)
			return null;

		int propIdx = expr.lastIndexOf(".");
		if (propIdx <= 0)
			return null;

		String propName = expr.substring(propIdx + 1, (expr.length() - 1));
		return propName;
	}

	

	/**
	 * Get session from faces context.
	 */
	public static HttpSession getSession()
	{
		return (HttpSession)FacesContext.getCurrentInstance().getExternalContext().getSession(false);
	}
    
    /**
     * Get attribute from session.
     */
    public static Object getSessionAttribute(String name)
    {
        return getSession().getAttribute(name);
    }

    /**
     * Set attribute in session.
     */
    public static void setSessionAttribute(String name, Object value)
    {
        getSession().setAttribute(name, value);
    }
    
    /**
     * Remove attribute from session.
     */
    public static void removeSessionAttribute(String name)
    {
        getSession().removeAttribute(name);
    }
    
	/**
	 * Get parameter value from request scope.
	 *
	 * @param name the name of the parameter
	 * @return the parameter value
	 */
	public static String getRequestParameter(String name)
	{
        return (String)getServletRequest().getParameter(name);
	}

    /**
     * Look for a request parameter with the given name.  If
     * it does not exist, look for a request attribute.
     * 
     * @param name The request parameter name to get.
     * @return The request value or null if not found. 
     */
    public static Object getRequestValue(String name)
    {
        Object param = getRequestParameter(name);
        if (param == null)
            param = getRequestAttribute(name);
        return param;
    }
    
    /**
     * Get attribute value from request scope.
     *
     * @param name the name of the parameter
     * @return the parameter value
     */
    public static Object getRequestAttribute(String name)
    {
        return getServletRequest().getAttribute(name);
    }
    
    /**
     * Set the attribute value in request scope.
     *
     * @param name the name of the parameter
     * @param value the parameter value
     */
    public static void setRequestAttribute(String name, Object value)
    {
        getServletRequest().setAttribute(name, value);
    }
    
   
	
	/**
	 * Add information message.
	 *
	 * @param msg the information message
	 */
	public static void addInfoMessage(String msg)
	{
		addInfoMessage(null, msg);
	}

	/**
	 * Add information message to a sepcific client.
	 *
	 * @param clientId the client id
	 * @param msg the information message
	 */
	public static void addInfoMessage(String clientId, String msg)
	{
		FacesContext.getCurrentInstance().addMessage(clientId, new FacesMessage(FacesMessage.SEVERITY_INFO, msg, msg));
	}

	/**
	 * Add error message.
	 *
	 * @param msg the error message
	 */
	public static void addErrorMessage(String msg)
	{
		addErrorMessage(null, msg);
	}

	/**
	 * Add error message to a specific client.
	 *
	 * @param clientId the client id
	 * @param msg the error message
	 */
	public static void addErrorMessage(String clientId, String msg)
	{
		FacesContext.getCurrentInstance().addMessage(clientId, new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));
	}

   
    
	/**
	   Returns true if the FacesContext has error messages in it, false if not.
	   @return true if the FacesContext has error messages in it, false if not.
	**/
	public static boolean hasErrorMessages()
	{
		for(Iterator itt = FacesContext.getCurrentInstance().getMessages(); itt.hasNext();)
		{
			FacesMessage nextMsg = (FacesMessage)itt.next();

			if ((nextMsg.getSeverity() == FacesMessage.SEVERITY_ERROR) ||
				(nextMsg.getSeverity() == FacesMessage.SEVERITY_FATAL))
				return true;
		}

		return false;
	}
    
	private static Application getApplication()
	{
		ApplicationFactory appFactory = (ApplicationFactory)FactoryFinder.getFactory(FactoryFinder.APPLICATION_FACTORY);
		return appFactory.getApplication();
	}

	/**
	   Returns the HttpServletReqeust object for the current request.
	   @return the HttpServletReqeust object for the current request.
	**/
	public static HttpServletRequest getServletRequest()
	{
		return (HttpServletRequest)FacesContext.getCurrentInstance().getExternalContext().getRequest();
	}

	/**
	   Returns the ValueBinding object for the specified EL expression.
	   @param el The EL expression.
	   @return the ValueBinding object for the specified EL expression.
	**/
	public static ValueBinding getValueBinding(String el)
	{
		return getApplication().createValueBinding(el);
	}
	   
    /**
	   Redirect to another page.  If there are any url parameters,
       strip them off of the url and place them into the request.
       
	   @param page The page to redirect to.
     */
    public static void redirectPage(String page)
    {
        // strip any url parameters and put them in request
        int paramStart = page.indexOf("?");
        if (paramStart != -1) {
            String paramString = page.substring(paramStart+1);
            String[] params = paramString.split("\\&");

            for (int i = 0; i < params.length; i++) {
                String[] paramParts = params[i].split("\\=");
                FacesUtils.setRequestAttribute(paramParts[0], paramParts[1]);    
            }
            
            page = page.substring(0, paramStart);
        }
        
        FacesContext context = FacesContext.getCurrentInstance();
        Application app = context.getApplication();
        UIViewRoot view = app.getViewHandler().createView(context, page);
        context.setViewRoot(view);
        context.renderResponse();  
    }
    
    private static String getJsfEl(String value)
    {
        return "#{" + value + "}";
    }
    
    /**
     * Return the id of the current view.
     */
    public static String getCurrentViewId()
    {
        return FacesContext.getCurrentInstance().getViewRoot().getViewId();
    }
}
